#ifndef ATLFCluster_H
#define ATLFCluster_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// ATLFast cluster                                                      //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TObject
#include <TObject.h>
#endif
#ifndef ROOT_TAtt3D
#include <TAtt3D.h>
#endif

class ATLFCluster : public TObject, public TAtt3D {

private:
   Int_t          m_KFcode;        //Cluster KF-code
   Int_t          m_Ncells;        //Number of cells used for reconstruction of this cluster
   Int_t          m_Nparticles;    //Number of particles assigned to this cluster
   Int_t          m_UseFlag;       //Flag for cluster usage
   Float_t        m_Eta0;          //Eta of initiator cell
   Float_t        m_Phi0;          //Phi of initiator cell
   Float_t        m_Eta;           //Eta of barry-center
   Float_t        m_Phi;           //Phi of barry-center
   Float_t        m_ET;            //PT of cluster

public:
                  ATLFCluster() {;}
                  ATLFCluster(Int_t code, Int_t ncells, Int_t nparticles, Int_t use, Float_t eta0, Float_t phi0,
                     Float_t eta, Float_t phi, Float_t et);
   virtual       ~ATLFCluster() {;}
   virtual char  *GetObjectInfo(Int_t px, Int_t py);
   virtual void   Paint(Option_t *option="");

//      Getters
   Int_t          KFcode()     {return m_KFcode;}
   Int_t          Ncells()     {return m_Ncells;}
   Int_t          Nparticles() {return m_Nparticles;}
   Int_t          UseFlag()    {return m_UseFlag;}
   Float_t        Eta0()       {return m_Eta0;}
   Float_t        Phi0()       {return m_Phi0;}
   Float_t        Eta()        {return m_Eta;}
   Float_t        Phi()        {return m_Phi;}
   Float_t        ET()         {return m_ET;}
   virtual void   Sizeof3D() const;

//      Setters
   void           SetEta0(Float_t eta) {m_Eta0 = eta;}
   void           SetPhi0(Float_t phi) {m_Phi0 = phi;}
   void           SetEta(Float_t eta) {m_Eta = eta;}
   void           SetPhi(Float_t phi) {m_Phi = phi;}
   void           SetET(Float_t et) {m_ET = et;}
   void           SetUseFlag(Int_t flag) {m_UseFlag = flag;}

   ClassDef(ATLFCluster, 1)   //ATLFast cluster
};

#endif
