#ifndef ATLFClusterMaker_H
#define ATLFClusterMaker_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// ATLFast ClusterMaker class.                                          //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ATLFMaker_H
#include "ATLFMaker.h"
#endif
#ifndef ROOT_TH1
#include <TH1.h>
#endif

class ATLFClusterMaker : public ATLFMaker {

protected:
   Int_t          m_Ncells;            //Number of cells
   Int_t          m_Nclusters;         //Number of clusters
   Float_t        m_MinETCalo;         //Minimum E_T for calo cluster
   Float_t        m_DeltaRBarrel1;     //Cone Delta R for barrel set 1
   Float_t        m_DeltaRForward1;    //Cone Delta R for forward set 1
   Float_t        m_DeltaRBarrel2;     //Cone Delta R for barrel set 2
   Float_t        m_DeltaRForward2;    //Cone Delta R for forward set 2
   Float_t        m_EtaCoverage;       //Rapidity coverage for calorimeter
   Float_t        m_MinETInitiator;    //Min E_T for cluster initiator
   Float_t        m_MinPTBfield;       //Min p_T for B-field
   Float_t        m_MinETCell;         //Min E_T for cell
   Float_t        m_BarrelForwardEta;  //Barrel/fwd transition in eta in calo
   Float_t        m_GranBarrelEta;     //Granularity in eta(barrel),2X in fwd
   Float_t        m_GranBarrelPhi;     //Granularity in phi(barrel),2X in fwd
   Bool_t         m_CellsEsharing;     //Cells energy sharing in clusters on/off

//     pointers to cell arrays
   Int_t         *m_CellIndex;         //!Internal index for reconstructed cells
   Int_t         *m_CellCount;         //!Number of particles for reconstructed cells
   Int_t         *m_CellFlag;          //!Flag usage for reconstructed cells
   Float_t       *m_CellEta;           //!Eta for reconstructed cells
   Float_t       *m_CellPhi;           //!Phi for reconstructed cells
   Float_t       *m_CellSumPT;         //!Sum of PTs for reconstructed cells

//     Clusters histograms
   TH1F          *m_Multiplicity;      //clusters multiplicity
   TH1F          *m_DeltaPhi;          //B-field delta_phi vers p_T
   TH1F          *m_Counters;          //B-field counters vers p_T
   TH1F          *m_PhiClu;            //delta_phi cluster-barry_centre_particle
   TH1F          *m_EtaClu;            //delta_eta cluster-barry_centre_particle
   TH1F          *m_DeltaR;            //delta_r cluster-barry_centre_particle

public:
                  ATLFClusterMaker();
                  ATLFClusterMaker(const char *name, const char *title);
   virtual       ~ATLFClusterMaker();
   virtual void   AddCluster(Int_t code, Int_t ncells, Int_t nparticles, Int_t use, Float_t eta0, Float_t phi0,
                             Float_t eta, Float_t phi, Float_t et);
   virtual void   Clear(Option_t *option="");
   Float_t        FieldPhi(Float_t pt, Float_t eta, Float_t theta);
   Float_t        HadronResolution(Int_t lumi, Float_t ene, Float_t eta, Float_t pt, Float_t rcone);
   virtual void   Finish();
   virtual void   Init();
   virtual Int_t  Make();
   virtual void   PrintInfo();
   virtual void   RemoveCluster(const Int_t cluster);

//    Getters 
   Float_t        MinETCalo() {return m_MinETCalo;}
   Float_t        DeltaRBarrel1() {return m_DeltaRBarrel1;}
   Float_t        DeltaRForward1() {return m_DeltaRForward1;}
   Float_t        DeltaRBarrel2() {return m_DeltaRBarrel2;}
   Float_t        DeltaRForward2() {return m_DeltaRForward2;}
   Float_t        EtaCoverage() {return m_EtaCoverage;}
   Float_t        MinETInitiator() {return m_MinETInitiator;}
   Float_t        MinPTBfield() {return m_MinPTBfield;}
   Float_t        MinETCell() {return m_MinETCell;}
   Float_t        BarrelForwardEta() {return m_BarrelForwardEta;}
   Float_t        GranBarrelEta() {return m_GranBarrelEta;}
   Float_t        GranBarrelPhi() {return m_GranBarrelPhi;}
   Bool_t         CellsEsharing() {return m_CellsEsharing;}

   Int_t          Ncells()    {return m_Ncells;}
   Int_t          Nclusters() {return m_Nclusters;}
   Int_t         *CellIndex() {return m_CellIndex;}
   Int_t         *CellCount() {return m_CellCount;}
   Int_t         *CellFlag()  {return m_CellFlag;}
   Float_t       *CellEta()   {return m_CellEta;}
   Float_t       *CellPhi()   {return m_CellPhi;}
   Float_t       *CellSumPT() {return m_CellSumPT;}

   TH1F          *Multiplicity() {return m_Multiplicity;}
   TH1F          *DeltaPhi()     {return m_DeltaPhi;}
   TH1F          *Counters()     {return m_Counters;}
   TH1F          *PhiClu()       {return m_PhiClu;}
   TH1F          *EtaClu()       {return m_EtaClu;}
   TH1F          *DeltaR()       {return m_DeltaR;}

//    Setters 
   void           SetMinETCalo(Float_t val=10.) {m_MinETCalo=val;}
   void           SetDeltaRBarrel1(Float_t val=0.4) {m_DeltaRBarrel1=val;}
   void           SetDeltaRForward1(Float_t val=0.4) {m_DeltaRForward1=val;}
   void           SetDeltaRBarrel2(Float_t val=0.7) {m_DeltaRBarrel2=val;}
   void           SetDeltaRForward2(Float_t val=0.7) {m_DeltaRForward2=val;}
   void           SetEtaCoverage(Float_t val=5.) {m_EtaCoverage=val;}
   void           SetMinETInitiator(Float_t val=1.5) {m_MinETInitiator=val;}
   void           SetMinPTBfield(Float_t val=0.5) {m_MinPTBfield=val;}
   void           SetMinETCell(Float_t val=0.) {m_MinETCell=val;}
   void           SetBarrelForwardEta(Float_t val=3.) {m_BarrelForwardEta=val;}
   void           SetGranBarrelEta(Float_t val=0.1) {m_GranBarrelEta=val;}
   void           SetGranBarrelPhi(Float_t val=0.1) {m_GranBarrelPhi=val;}
   void           SetCellsEsharing(Bool_t val=0) {m_CellsEsharing=val;}

   ClassDef(ATLFClusterMaker, 1)   //ATLFast ClusterMaker
};

#endif
