#ifndef ATLFDisplay_H
#define ATLFDisplay_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// ATLFDisplay                                                          //
//                                                                      //
// Utility class to display ATLAS outline, tracks, clusters, jets,..    //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ATLFVirtualDisplay_H
#include "ATLFVirtualDisplay.h"
#endif

#ifndef ROOT_TCanvas
#include <TCanvas.h>
#endif
#ifndef ROOT_TNode
#include <TNode.h>
#endif
#ifndef ROOT_TArc
#include <TArc.h>
#endif
#ifndef ROOT_TTUBE
#include <TTUBE.h>
#endif

class ATLFTrigger;
class TPolyLine3D;
class ATLFParticle;

class ATLFDisplay : public ATLFVirtualDisplay {

private:
   Bool_t            m_DrawAllViews;         //Flag True if AllViews selected
   Bool_t            m_DrawParticles;        //Flag True if particles to be drawn
   Bool_t            m_DrawClusters;         //Flag True if clusters to be drawn
   Float_t           m_PTcut;                //PT cut to display objects
   Float_t           m_PTcutEGMUNU;          //PT cut for Electrons, Gammas, MUons, Neutrinos
   Float_t           m_Rin;                  //Inner ATLAS radius
   Float_t           m_Rout;                 //Outer ATLAS radius
   Float_t           m_Zin;                  //Inner ATLAS length along Z
   Float_t           m_Zout;                 //Outer ATLAS length along Z
   Float_t           m_Theta;                //Viewing angle theta
   Float_t           m_Phi;                  //Viewing angle phi
   TCanvas          *m_Canvas;               //Pointer to the display canvas
   TPad             *m_TrigPad;              //Pointer to the trigger pad 
   TPad             *m_Buttons;              //Pointer to the buttons pad
   TPad             *m_Pad;                  //Pointer to the event display main pad
   TArc             *m_EM1;                  //Pointer to arc showing ON/OFF trigger EM1
   TArc             *m_PH1;                  //Pointer to arc showing ON/OFF trigger PH1
   TArc             *m_EM2;                  //Pointer to arc showing ON/OFF trigger EM2
   TArc             *m_MU1;                  //Pointer to arc showing ON/OFF trigger MU1
   TArc             *m_MU2;                  //Pointer to arc showing ON/OFF trigger MU2
   TArc             *m_EMU;                  //Pointer to arc showing ON/OFF trigger EMU
   TArc             *m_JT1;                  //Pointer to arc showing ON/OFF trigger JT1
   TArc             *m_JT3;                  //Pointer to arc showing ON/OFF trigger JT3
   TArc             *m_JT4;                  //Pointer to arc showing ON/OFF trigger JT4
   TArc             *m_ALL;                  //Pointer to arc showing ON/OFF trigger ALL
   TTUBE            *m_Tubin;                //Inner tube
   TTUBE            *m_Tubout;               //outer tube
   TNode            *m_Nodin;                //Node for detector outline
   TList            *m_Fruits;               //List for fruits
   ATLFParticle     *m_Particle;             //Pointer to Particle graphics manager
   
public:
                     ATLFDisplay();
                     ATLFDisplay(const char *title);
   virtual          ~ATLFDisplay();
   virtual Bool_t    AllViews() {return m_DrawAllViews;}
   virtual void      Clear(Option_t *option="");
   virtual void      DisplayButtons();
   virtual void      DisplayTrigger(ATLFTrigger *trig);
   virtual Int_t     DistancetoPrimitive(Int_t px, Int_t py);
   virtual void      Draw(Option_t *option="");
   virtual void      DrawAllViews();
   Bool_t            DrawClusters() {return m_DrawClusters;}
   Bool_t            DrawParticles() {return m_DrawParticles;}
   virtual void      DrawTitle(Option_t *option="");
   virtual void      DrawView(Float_t theta, Float_t phi);
   virtual void      DrawViewGL();
   virtual void      DrawViewX3D();
   virtual void      ExecuteEvent(Int_t event, Int_t px, Int_t py);
   virtual void      GetEvent(Int_t event); // *MENU*
   TNode            *Nodin() {return m_Nodin;}
   TTUBE            *Tubin() {return m_Tubin;}
   TPad             *Pad() {return m_Pad;}
   virtual void      Paint(Option_t *option="");
   virtual void      PaintFruit(TObject *obj, Float_t eta, Float_t phi, Float_t pt, Int_t type, Option_t *option="");
   virtual void      PaintParticles(Option_t *option="");
   Float_t           PTcut() {return m_PTcut;}
   Float_t           PTcutEGMUNU() {return m_PTcutEGMUNU;}
   Float_t           Rin() {return m_Rin;}
   Float_t           Rout() {return m_Rout;}
   virtual void      SetDrawClusters(Bool_t draw=kTRUE) {m_DrawClusters=draw;}   // *MENU*
   virtual void      SetDrawParticles(Bool_t draw=kTRUE) {m_DrawParticles=draw;} // *MENU*
   virtual void      SetPTcut(Float_t ptcut=0.4); // *MENU*
   virtual void      SetPTcutEGMUNU(Float_t ptcut=5); // *MENU*
   virtual void      SetGeometry(Float_t rin=115); // *MENU*
   virtual void      SetView(Float_t theta, Float_t phi);
   virtual void      ShowNextEvent(Int_t delta=1);
   virtual void      SizeFruit() const;
   virtual void      SizeParticles() const;
   Float_t           Zin() {return m_Zin;}
   Float_t           Zout() {return m_Zout;}

   ClassDef(ATLFDisplay, 1)   //Utility class to display ATLAS outline, tracks, clusters, jets,..
};

#endif
