#ifndef ATLFJet_H
#define ATLFJet_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// ATLFast jet class                                                    //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TObject
#include <TObject.h>
#endif
#ifndef ROOT_TAtt3D
#include <TAtt3D.h>
#endif

class ATLFJet : public TObject, public TAtt3D {

private:
   Int_t          m_KFcode;        //Jet KF-code
   Int_t          m_Ncells;        //Number of cells used for reconstruction
   Int_t          m_Nparticles;    //Number of particles assigned to jet
   Int_t          m_Part;          //Position in MCParticle list of matching b-quark/c-quark) after FSR or tau
   Float_t        m_Eta0;          //Eta position of initiator cell
   Float_t        m_Phi0;          //Phi position of initiator cell
   Float_t        m_Eta;           //Eta  of jet bary-center
   Float_t        m_Phi;           //Phi  of jet bary-center
   Float_t        m_PT;            //Transverse momentum of jet

public:
                  ATLFJet() {;}
                  ATLFJet(Int_t code, Int_t ncells, Int_t nparticles, Int_t part,
                        Float_t eta0, Float_t phi0, Float_t eta, Float_t phi, Float_t pt);
   virtual       ~ATLFJet() {;}
   virtual char  *GetObjectInfo(Int_t px, Int_t py);
   virtual void   Paint(Option_t *option="");
   Int_t          KFcode() {return m_KFcode;}
   Int_t          Part()   {return m_Part;}
   Float_t        Eta0()   {return m_Eta0;}
   Float_t        Phi0()   {return m_Phi0;}
   Float_t        Eta()    {return m_Eta;}
   Float_t        Phi()    {return m_Phi;}
   Float_t        PT()     {return m_PT;}
   void           SetKFcode(Int_t code) {m_KFcode = code;}
   void           SetPart(Int_t part)   {m_Part = part;}
   void           SetPT(Float_t pt)     {m_PT  = pt;}
   virtual void   Sizeof3D() const;

   ClassDef(ATLFJet, 1)   //ATLFast jet class
};

#endif
