#ifndef ATLFMisc_H
#define ATLFMisc_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// ATLFast miscellaneous class                                          //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TObject
#include <TObject.h>
#endif
#ifndef ROOT_TAtt3D
#include <TAtt3D.h>
#endif

class ATLFMisc : public TObject, public TAtt3D {

private:
// Event identification
   Int_t          m_Run;              //Run number
   Int_t          m_Event;            //Event number
   Int_t          m_MCProcess;        //Code for hard scattering process
// Event characteristic 
   Int_t          m_Nelectrons;       //Isolated electrons
   Int_t          m_Nmuons;           //Isolated muons
   Int_t          m_Nmuonsx;          //NonIsolated muons
   Int_t          m_Nphotons;         //Isolated photons
   Int_t          m_Nalljets;         //All jets
   Int_t          m_Nbjets;           //b-labelled jets
   Int_t          m_Ncjets;           //c-labelled jets
   Int_t          m_Ntaujets;         //tau-labelled jets
// misc
   Float_t        m_CircJets;        //Circularity from jets
   Float_t        m_CircEvent;       //Circularity from cells
   Float_t        m_Thrust;          //Event Thrust
   Float_t        m_Oblateness;      //Event Oblateness
// missing energy
   Float_t        m_pxmiss;          //Reconstructed missing momentum along X
   Float_t        m_pymiss;          //Reconstructed missing momentum along Y
   Float_t        m_pxnu;            //Sum of X momenta components escaping detection
   Float_t        m_pynu;            //Sum of Y momenta components escaping detection

public:
                  ATLFMisc();
   virtual       ~ATLFMisc() {;}

   Float_t        CircJets()   {return m_CircJets;}
   Float_t        CircEvent()  {return m_CircEvent;}
   virtual char  *GetObjectInfo(Int_t px, Int_t py);
   Float_t        Thrust()     {return m_Thrust;}
   Float_t        Oblateness() {return m_Oblateness;}
   Float_t        pxmiss()     {return m_pxmiss;}
   Float_t        pymiss()     {return m_pymiss;}
   Int_t          Nelectrons() {return m_Nelectrons;}
   Int_t          Nmuons()   {return m_Nmuons;}
   Int_t          Nmuonsx()  {return m_Nmuonsx;}
   Int_t          Nphotons() {return m_Nphotons;}
   Int_t          Nalljets() {return m_Nalljets;}
   Int_t          Nbjets()   {return m_Nbjets;}
   Int_t          Ncjets()   {return m_Ncjets;}
   Int_t          Ntaujets() {return m_Ntaujets;}
   virtual void   Paint(Option_t *option="");
   virtual void   Sizeof3D() const;

   virtual void   SetMissing(Float_t pxmiss, Float_t pymiss, Float_t pxnu, Float_t pynu);
   virtual void   SetCircJets(Float_t val) {m_CircJets = val;}
   virtual void   SetCircEvent(Float_t val) {m_CircEvent = val;}
   virtual void   SetThrust(Float_t val) {m_Thrust = val;}
   virtual void   SetOblateness(Float_t val) {m_Oblateness = val;}

   virtual void   SetRun(Int_t run=1) {m_Run=run;}
   virtual void   SetEvent(Int_t event=1) {m_Event=event;}
   virtual void   SetMCProcess(Int_t process=0) {m_MCProcess=process;}
   virtual void   SetNelectrons(Int_t nele=0) {m_Nelectrons=nele;}
   virtual void   SetNmuons(Int_t nmuo=0) {m_Nmuons=nmuo;}
   virtual void   SetNmuonsx(Int_t nmuox=0) {m_Nmuonsx=nmuox;}
   virtual void   SetNphotons(Int_t nphot=0) {m_Nphotons=nphot;}
   virtual void   SetNalljets(Int_t njet=0) {m_Nalljets=njet;}
   virtual void   SetNbjets(Int_t nbjet=0) {m_Nbjets=nbjet;}
   virtual void   SetNcjets(Int_t ncjet=0) {m_Ncjets=ncjet;}
   virtual void   SetNtaujets(Int_t ntaujet=0) {m_Ntaujets=ntaujet;}

   ClassDef(ATLFMisc, 1)   //ATLFast miscellaneous class
};

#endif
