#ifndef ATLFMiscMaker_H
#define ATLFMiscMaker_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// ATLFast MiscMaker class.                                             //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ATLFMaker_H
#include "ATLFMaker.h"
#endif
#ifndef ROOT_TH1
#include <TH1.h>
#endif

class ATLFMiscMaker : public ATLFMaker {

private:
   Float_t        m_MinET;         //Minimum E_T for unused cells

//     Misc Energy histograms
   TH1F          *m_CircJets;      //circularity of jets
   TH1F          *m_CircEvent;     //circularity of event
   TH1F          *m_Thrust;        //thrust of  event
   TH1F          *m_Oblateness;    //oblateness of event
//     Missing Energy histograms
   TH1F          *m_RecPT;         //reconstructed p_T
   TH1F          *m_RecPTcells;    //reconstructed p_T +cells
   TH1F          *m_PTnu;          //p_T nu
   TH1F          *m_PXnu;          //pX nu
   TH1F          *m_PYnu;          //pY nu
   TH1F          *m_RecPX;         //reconstructed pX
   TH1F          *m_RecPY;         //reconstructed pY
   TH1F          *m_RecPXcells;    //reconstructed pX + cells
   TH1F          *m_RecPYcells;    //reconstructed pY + cells
   TH1F          *m_ResolPX;       //resol missing pX
   TH1F          *m_ResolPX0;      //resol missing pX (sumET=0-25)
   TH1F          *m_ResolPX25;     //resol missing pX (sumET=25-50)
   TH1F          *m_ResolPX50;     //resol missing pX (sumET=50-100)
   TH1F          *m_ResolPX100;    //resol missing pX (sumET=100-200)
   TH1F          *m_ResolPX200;    //resol missing pX (sumET=200-300)
   TH1F          *m_ResolPX300;    //resol missing pX (sumET=300-400)
   TH1F          *m_PTmissPTnu;    //PTmiss/PTnu
   TH1F          *m_PXmissPXnu;    //PXmiss/PXnu
   TH1F          *m_PYmissPYnu;    //PYmiss/PYnu


public:
                  ATLFMiscMaker();
                  ATLFMiscMaker(const char *name, const char *title);
   virtual       ~ATLFMiscMaker();
           void   Circularity(Float_t &circjet, Float_t &circeve);
   virtual void   Clear(Option_t *option="");
   virtual void   Finish();
   virtual void   Init();
   virtual Int_t  Make();
   virtual void   MakeMisc();
   virtual void   MakeMissing();
   virtual void   PrintInfo();
           void   ThrustOblateness(Float_t &thrust, Float_t &obl);
           void   EigenValues(Int_t nm, Int_t n, Float_t ar[2][2], Float_t *wr, Float_t zr[2][2],
                           Int_t &ierr, Float_t *work);
           void   EigenValue1(Int_t nm, Int_t n, Float_t a[2][2], Float_t *d, Float_t *e, Float_t z[2][2]);
           void   EigenValue2(Int_t nm, Int_t n, Float_t *d, Float_t *e, Float_t z[2][2], Int_t &ierr);
           void   Atlut3(Int_t n,Float_t p[][5], Float_t &thr, Float_t &obl);
           void   Atanxy(Float_t xx, Float_t yy, Float_t &ang, Int_t &ierr);
#ifdef NEW
           void   Atlth4(Int_t ntrak, Int_t itkdm, Float_t *ptrak, Float_t *thrval, Float_t thrvec[][3], Int_t &ierr);
#endif
           void   Atplu3(Int_t indx, Float_t plund[][5], Float_t *pvec, const char *option);
           void   Atrob3(Float_t rmx[3][3], Float_t *vect, Float_t *rvec);
           void   Atrof3(Float_t rmx[3][3], Float_t *vect, Float_t *rvec);
           void   Atrmx3(Float_t *vect, Float_t cp, Float_t sp, Float_t rmx[3][3]);

   Float_t        MinET() {return m_MinET;}
   void           SetMinET(Float_t val=0.) {m_MinET=val;}

   TH1F          *CircJets()    {return m_CircJets;}
   TH1F          *CircEvent()   {return m_CircEvent;}
   TH1F          *Thrust()      {return m_Thrust;}
   TH1F          *Oblateness()  {return m_Oblateness;}

   TH1F          *RecPT()        {return m_RecPT;}
   TH1F          *RecPTcells()   {return m_RecPTcells;}
   TH1F          *PTnu()         {return m_PTnu;}
   TH1F          *PXnu()         {return m_PXnu;}
   TH1F          *PYnu()         {return m_PYnu;}
   TH1F          *RecPX()        {return m_RecPX;}
   TH1F          *RecPY()        {return m_RecPY;}
   TH1F          *RecPXcells()   {return m_RecPXcells;}
   TH1F          *RecPYcells()   {return m_RecPYcells;}
   TH1F          *ResolPX()      {return m_ResolPX;}
   TH1F          *ResolPX0()     {return m_ResolPX0;}
   TH1F          *ResolPX25()    {return m_ResolPX25;}
   TH1F          *ResolPX50()    {return m_ResolPX50;}
   TH1F          *ResolPX100()   {return m_ResolPX100;}
   TH1F          *ResolPX200()   {return m_ResolPX200;}
   TH1F          *ResolPX300()   {return m_ResolPX300;}
   TH1F          *PTmissPTnu()   {return m_PTmissPTnu;}
   TH1F          *PXmissPXnu()   {return m_PXmissPXnu;}
   TH1F          *PYmissPYnu()   {return m_PYmissPYnu;}

   ClassDef(ATLFMiscMaker, 1)   //ATLFast MiscMaker
};

#endif
