#ifndef ATLFTrack_H
#define ATLFTrack_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// ATLFast track class                                                  //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TObject
#include <TObject.h>
#endif

class ATLFTrack : public TObject {

private:
   Int_t          m_KFcode;        //Particle KF-code
   Int_t          m_MCParticle;    //Particle position in MCParticles list
   Int_t          m_Nmothers;      //Number of mothers
   Int_t          m_KFmother[6];   //KF-code of up to 6th mother
   Int_t          m_MCmother[6];   //Particle number in MCParticles list of Ith mother
   Float_t        m_a0;            //track parameters after smearing
   Float_t        m_z0;            //track parameters after smearing
   Float_t        m_phi;           //track parameters after smearing
   Float_t        m_cot;           //track parameters after smearing
   Float_t        m_PTinv;         //track parameters after smearing
   Float_t        m_a0Crude;       //track parameters before smearing
   Float_t        m_z0Crude;       //track parameters before smearing
   Float_t        m_phiCrude;      //track parameters before smearing
   Float_t        m_cotCrude;      //track parameters before smearing
   Float_t        m_PTinvCrude;    //track parameters before smearing
   Float_t        m_CORR[25];      //Correlation coeeficients

public:
                  ATLFTrack() {;}
                  ATLFTrack(Int_t code, Int_t mcparticle);
   virtual       ~ATLFTrack() {;}
   virtual void   Draw(Option_t *option="");
   Int_t          KFcode() {return m_KFcode;}
   Int_t          MCParticle() {return m_MCParticle;}
   Int_t          KFmother(Int_t i) {return m_KFmother[i];}
   Int_t          MCmother(Int_t i) {return m_MCmother[i];}
   Float_t        PTinv() {return m_PTinv;}
   virtual void   GetSmearedTrack(Float_t &a0, Float_t &z0, Float_t &phi, Float_t &cot, Float_t &ptinv);
   virtual void   GetCrudeTrack(Float_t &a0, Float_t &z0, Float_t &phi, Float_t &cot, Float_t &ptinv);
   Float_t       *GetCorrelations() {return &m_CORR[0];}
   virtual void   Paint(Option_t *option="");
   virtual void   SetMother(Int_t i, Int_t code, Int_t mcparticle);
   virtual void   SetSmearedTrack(Float_t a0, Float_t z0, Float_t phi, Float_t cot, Float_t ptinv);
   virtual void   SetCrudeTrack(Float_t a0, Float_t z0, Float_t phi, Float_t cot, Float_t ptinv);
   virtual void   SetCorrelations(Float_t *corr);

   ClassDef(ATLFTrack, 1)   //ATLFast track class
};

#endif
