#ifndef ATLFTriggerMaker_H
#define ATLFTriggerMaker_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// ATLFast TriggerMaker class.                                          //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ATLFMaker_H
#include "ATLFMaker.h"
#endif
#ifndef ROOT_TH1
#include <TH1.h>
#endif

class ATLFTriggerMaker : public ATLFMaker {

protected:
//    Selected trigger parameters from the current luminosity
   Float_t        m_PTELE;            //Minimum p_T for 1 electron
   Float_t        m_PTPHO;            //Minimum p_T for 1 photon
   Float_t        m_PTEM2;            //Minimum p_T for 2 ele/phot
   Float_t        m_PTMU1I;           //Minimum p_T for 1 iso muon
   Float_t        m_PTMU1;            //Minimum p_T for 1 muon
   Float_t        m_PTMU2;            //Minimum p_T for 2 muons
   Float_t        m_PTMU2I;           //Minimum p_T for 2 iso muons
   Float_t        m_ETAEL1;           //eta-coverage for electrons
   Float_t        m_ETAMU1;           //eta-coverage for muons
   Float_t        m_PTEMUE;           //Minimum p_T for iso electron
   Float_t        m_PTEMUM;           //Minimum p_T for muon
   Float_t        m_PTJET1;           //Minimum p_T for 1 jet
   Float_t        m_PTJET3;           //Minimum p_T for 3 jets
   Float_t        m_PTJET4;           //Minimum p_T for 4 jets
//    parameters for isolated electron/photon trigger
   Float_t        m_EPhoEtaCoverage;  //eta-coverage for electrons/photons
   Float_t        m_EPhoMinPTlowEle;  //Minimum p_T for 1 electron low lumi
   Float_t        m_EPhoMinPThighEle; //Minimum p_T for 1 electron high lumi
   Float_t        m_EPhoMinPTlowPho;  //Minimum p_T for 1 photon low lumi
   Float_t        m_EPhoMinPThighPho; //Minimum p_T for 1 photon high lumi
   Float_t        m_EPhoMinPTlow2;    //Minimum p_T for 2 ele/phot low lumi
   Float_t        m_EPhoMinPThigh2;   //Minimum p_T for 2 ele/phot high lumi
//    parameters for muon trigger
   Float_t        m_MuoEtaCoverage;   //eta coverage for muons   
   Float_t        m_MuoMinPTlowIso;   //Minimum p_T for 1 iso muon low lumi
   Float_t        m_MuoMinPThighIso;  //Minimum p_T for 1 iso muon high lumi
   Float_t        m_MuoMinPTlow;      //Minimum p_T for 1 muon low lumi
   Float_t        m_MuoMinPThigh;     //Minimum p_T for 1 muon high lumi
   Float_t        m_MuoMinPTlow2;     //Minimum p_T for 2 muons low lumi
   Float_t        m_MuoMinPThigh2;    //Minimum p_T for 2 muons high lumi
   Float_t        m_MuoMinPThigh2Iso; //Minimum p_T for 2 iso muons high lumi
//    parameters for electron/muon trigger
   Float_t        m_EMuoEtaCoverageE; //eta-coverage for electrons
   Float_t        m_EMuoEtaCoverageM; //eta-coverage for muons
   Float_t        m_EMuoMinPTlowEle;  //Minimum p_T for iso electron low lumi
   Float_t        m_EMuoMinPThighEle; //Minimum p_T for iso electron high lumi
   Float_t        m_EMuoMinPTlowMuo;  //Minimum p_T for muon low lumi
   Float_t        m_EMuoMinPThighMuo; //Minimum p_T for muon high lumi
//    parameters for jet trigger
   Float_t        m_JetEtaCoverage;   //eta coverage for jets
   Float_t        m_JetMinPTlow1jet;  //Minimum p_T for 1 jet low lumi
   Float_t        m_JetMinPThigh1jet; //Minimum p_T for 1 jet high lumi
   Float_t        m_JetMinPTlow3jet;  //Minimum p_T for 3 jets low lumi
   Float_t        m_JetMinPThigh3jet; //Minimum p_T for 3 jets high lumi
   Float_t        m_JetMinPTlow4jet;  //Minimum p_T for 4 jets low lumi
   Float_t        m_JetMinPThigh4jet; //Minimum p_T for 4 jets high lumi
//  trigger histograms
   TH1F          *m_Content;          //trigger content
   TH1F          *m_MuonEffHighLow;   //muon trig eff. for high + low  pT threshold 
   TH1F          *m_MuonEffLow;       //muon trig eff. for low pT threshold
   TH1F          *m_MuonEffHigh;      //muon trig eff. for high pT threshold
   TH1F          *m_Counter;          //muon trig counter


public:
                  ATLFTriggerMaker();
                  ATLFTriggerMaker(const char *name, const char *title);
   virtual       ~ATLFTriggerMaker();
   virtual void   Clear(Option_t *option="");
   virtual void   Finish();
   virtual void   Init();
   virtual Int_t  Make();
   virtual void   PrintInfo();

//    Getters for isolated electron/photon trigger
   Float_t        EPhoEtaCoverage() {return m_EPhoEtaCoverage;}
   Float_t        EPhoMinPTlowEle() {return m_EPhoMinPTlowEle;}
   Float_t        EPhoMinPThighEle() {return m_EPhoMinPThighEle;}
   Float_t        EPhoMinPTlowPho() {return m_EPhoMinPTlowPho;}
   Float_t        EPhoMinPThighPho() {return m_EPhoMinPThighPho;}
   Float_t        EPhoMinPTlow2() {return m_EPhoMinPTlow2;}
   Float_t        EPhoMinPThigh2() {return m_EPhoMinPThigh2;}
//    Getters for muon trigger
   Float_t        MuoEtaCoverage() {return m_MuoEtaCoverage;}
   Float_t        MuoMinPTlowIso() {return m_MuoMinPTlowIso;}
   Float_t        MuoMinPThighIso() {return m_MuoMinPThighIso;}
   Float_t        MuoMinPTlow() {return m_MuoMinPTlow;}
   Float_t        MuoMinPThigh() {return m_MuoMinPThigh;}
   Float_t        MuoMinPTlow2() {return m_MuoMinPTlow2;}
   Float_t        MuoMinPThigh2() {return m_MuoMinPThigh2;}
   Float_t        MuoMinPThigh2Iso() {return m_MuoMinPThigh2Iso;}
//    Getters for electron/muon trigger
   Float_t        EMuoEtaCoverageE() {return m_EMuoEtaCoverageE;}
   Float_t        EMuoEtaCoverageM() {return m_EMuoEtaCoverageM;}
   Float_t        EMuoMinPTlowEle() {return m_EMuoMinPTlowEle;}
   Float_t        EMuoMinPThighEle() {return m_EMuoMinPThighEle;}
   Float_t        EMuoMinPTlowMuo() {return m_EMuoMinPTlowMuo;}
   Float_t        EMuoMinPThighMuo() {return m_EMuoMinPThighMuo;}
//    Getters for jet trigger
   Float_t        JetEtaCoverage() {return m_JetEtaCoverage;}
   Float_t        JetMinPTlow1jet() {return m_JetMinPTlow1jet;}
   Float_t        JetMinPThigh1jet() {return m_JetMinPThigh1jet;}
   Float_t        JetMinPTlow3jet() {return m_JetMinPTlow3jet;}
   Float_t        JetMinPThigh3jet() {return m_JetMinPThigh3jet;}
   Float_t        JetMinPTlow4jet() {return m_JetMinPTlow4jet;}
   Float_t        JetMinPThigh4jet() {return m_JetMinPThigh4jet;}

//  Getters for trigger histograms
   TH1F          *Content()        {return m_Content;}
   TH1F          *MuonEffHighLow() {return m_MuonEffHighLow;}
   TH1F          *MuonEffLow()     {return m_MuonEffLow;}
   TH1F          *MuonEffHigh()    {return m_MuonEffHigh;}
   TH1F          *Counter()        {return m_Counter;}

//    Setters for isolated electron/photon trigger
   void           SetEPhoEtaCoverage(Float_t val=2.5) {m_EPhoEtaCoverage=val;}
   void           SetEPhoMinPTlowEle(Float_t val=20.) {m_EPhoMinPTlowEle=val;}
   void           SetEPhoMinPThighEle(Float_t val=30.) {m_EPhoMinPThighEle=val;}
   void           SetEPhoMinPTlowPho(Float_t val=40.) {m_EPhoMinPTlowPho=val;}
   void           SetEPhoMinPThighPho(Float_t val=60.) {m_EPhoMinPThighPho=val;}
   void           SetEPhoMinPTlow2(Float_t val=15.) {m_EPhoMinPTlow2=val;}
   void           SetEPhoMinPThigh2(Float_t val=20.) {m_EPhoMinPThigh2=val;}
//    Setters for muon trigger
   void           SetMuoEtaCoverage(Float_t val=2.4) {m_MuoEtaCoverage=val;}
   void           SetMuoMinPTlowIso(Float_t val=6.) {m_MuoMinPTlowIso=val;}
   void           SetMuoMinPThighIso(Float_t val=20.) {m_MuoMinPThighIso=val;}
   void           SetMuoMinPTlow(Float_t val=15.) {m_MuoMinPTlow=val;}
   void           SetMuoMinPThigh(Float_t val=30.) {m_MuoMinPThigh=val;}
   void           SetMuoMinPTlow2(Float_t val=6.) {m_MuoMinPTlow2=val;}
   void           SetMuoMinPThigh2(Float_t val=10.) {m_MuoMinPThigh2=val;}
   void           SetMuoMinPThigh2Iso(Float_t val=6.) {m_MuoMinPThigh2Iso=val;}
//    Setters for electron/muon trigger
   void           SetEMuoEtaCoverageE(Float_t val=2.5) {m_EMuoEtaCoverageE=val;}
   void           SetEMuoEtaCoverageM(Float_t val=2.4) {m_EMuoEtaCoverageM=val;}
   void           SetEMuoMinPTlowEle(Float_t val=5.) {m_EMuoMinPTlowEle=val;}
   void           SetEMuoMinPThighEle(Float_t val=15.) {m_EMuoMinPThighEle=val;}
   void           SetEMuoMinPTlowMuo(Float_t val=6.) {m_EMuoMinPTlowMuo=val;}
   void           SetEMuoMinPThighMuo(Float_t val=6.) {m_EMuoMinPThighMuo=val;}
//    Setters for jet trigger
   void           SetJetEtaCoverage(Float_t val=3.2) {m_JetEtaCoverage=val;}
   void           SetJetMinPTlow1jet(Float_t val=300.) {m_JetMinPTlow1jet=val;}
   void           SetJetMinPThigh1jet(Float_t val=300.) {m_JetMinPThigh1jet=val;}
   void           SetJetMinPTlow3jet(Float_t val=80.) {m_JetMinPTlow3jet=val;}
   void           SetJetMinPThigh3jet(Float_t val=100.) {m_JetMinPThigh3jet=val;}
   void           SetJetMinPTlow4jet(Float_t val=30.) {m_JetMinPTlow4jet=val;}
   void           SetJetMinPThigh4jet(Float_t val=50.) {m_JetMinPThigh4jet=val;}

   ClassDef(ATLFTriggerMaker, 1)   //ATLFast TriggerMaker
};

#endif
