// macro to read Resolution data from an ascii file 
// and convert to a Root Histogram 
// Author: Leanne Guy

void TrackResData()
{

   gROOT->Reset();

   // Open Ascii File
   FILE *fp = fopen("IDResolMu011.dat","r");
   if (fp <= 0) printf("Sorry, cannot open file \n");


   // Open Root Output file 
   TFile *outFile = new TFile("IDResolMu011.root","RECREATE");

   // Scan in data

   char comment[100];
   fgets(&comment,50,fp);

   // Number of Eta Bins
   int nEta = 0;
   fscanf(fp,"%d\n",&nEta);

   fgets(&comment,50,fp);

   // Eta Bin values
   int k = 1;
   TH1F *EtaValues = new TH1F("EtaValues","Eta Values",nEta+1,1,nEta+2);
   float eta1,eta2, eta3, eta4,eta5;
   for (int i=0; i<5; i++) {
     fscanf(fp,"%f %f %f %f %f\n",&eta1,&eta2,&eta3,&eta4,&eta5);
     EtaValues->Fill(k,eta1);
     EtaValues->Fill(k+1,eta2);
     EtaValues->Fill(k+2,eta3);
     EtaValues->Fill(k+3,eta4);
     EtaValues->Fill(k+4,eta5);
     k +=5;
   }
   // Last values
   fscanf(fp,"%f\n",&eta1);
   EtaValues->Fill(k,eta1);
   EtaValues->Fill(k+1,eta1+0.01);

   fgets(&comment,50,fp);

   // Number of Pt Bins
   int nPt;
   fscanf(fp,"%d\n",&nPt);

   fgets(&comment,50,fp);

   // Pt Bin values
    int k = 1;
    TH1F *PtValues = new TH1F("PtValues","Pt Values",nPt+1,1,nPt+2);
    float pt1,pt2,pt3,pt4,pt5;
    for (int i=0; i<2; i++) {
     fscanf(fp,"%f %f %f %f %f\n",&pt1,&pt2,&pt3,&pt4,&pt5);
     PtValues->Fill(k,pt1);
     PtValues->Fill(k+1,pt2);
     PtValues->Fill(k+2,pt3);
     PtValues->Fill(k+3,pt4);
     PtValues->Fill(k+4,pt5);
     k +=5;
    }

    // Last values
   fscanf(fp,"%f %f %f %f\n",&pt1,&pt2,&pt3,&pt4);
   PtValues->Fill(k,pt1);
   PtValues->Fill(k+1,pt2);
   PtValues->Fill(k+2,pt3);
   PtValues->Fill(k+3,pt4);
   PtValues->Fill(k+4,pt4+0.01);


   // Array for axis values
   Float_t PtValuesA[nPt+2];
   Float_t EtaValuesA[nEta+2];

   for (int i = 0; i<nPt+1; i++) {
      PtValuesA[i] = PtValues->GetBinContent(i+1);
   }

   for (int i = 0; i<nEta+1; i++) {
      EtaValuesA[i] = EtaValues->GetBinContent(i+1);
   }

   // 2D Histograms for storing resoultion information
   TH2F *TvIP     = new TH2F("TvIP","Transverse IP",nEta,EtaValuesA,nPt,PtValuesA);
   TH2F *LtIP     = new TH2F("LtIP","Longitudinal IP",nEta,EtaValuesA,nPt,PtValuesA);
   TH2F *Phi0     = new TH2F("Phi0","Phi0",nEta,EtaValuesA,nPt,PtValuesA);
   TH2F *CoTanT   = new TH2F("CoTanT","Cotan Theta",nEta,EtaValuesA,nPt,PtValuesA);
   TH2F *QdPT     = new TH2F("QdPT","Charge/PT",nEta,EtaValuesA,nPt,PtValuesA);
   TH2F *InvPtPhi = new TH2F("InvPtPhi","Inv(Pt-Phi)",nEta,EtaValuesA,nPt,PtValuesA);
   TH2F *InvPtA0 = new TH2F("InvPtA0","Inv(Pt-A0)",nEta,EtaValuesA,nPt,PtValuesA);
   TH2F *PhiA0   = new TH2F("PhiA0","(Phi - A0)",nEta,EtaValuesA,nPt,PtValuesA);
   TH2F *CoTTZ0  = new TH2F("CoTTZ0","cot(theta)-z0",nEta,EtaValuesA,nPt,PtValuesA);

   fgets(&comment,100,fp);
   fgets(&comment,100,fp);
 
   // Scan in resolution data
   Float_t cv1,cv2,cv3,cv4,cv5,cv6,cv7,cv8,cv9;
   for (int i=0; i<nEta; i++ ) {
      Float_t eta = EtaValues->GetBinContent(i+1) + 0.001;
      for (int j=0; j<nPt; j++ ) {
         Float_t pt = PtValues->GetBinContent(j+1) + 0.001;
         fscanf(fp,"%f %f %f %f %f\n",&cv1,&cv2,&cv3,&cv4,&cv5);
         fscanf(fp,"%f %f %f %f\n",&cv6,&cv7,&cv8,&cv9);
         TvIP->Fill(eta,pt,cv1);
         LtIP->Fill(eta,pt,cv2);
         Phi0->Fill(eta,pt,cv3);
         CoTanT->Fill(eta,pt,cv4);
         QdPT->Fill(eta,pt,cv5);
         InvPtPhi->Fill(eta,pt,cv6);
         InvPtA0->Fill(eta,pt,cv7);
         PhiA0->Fill(eta,pt,cv8);
         CoTTZ0->Fill(eta,pt,cv9);
      }
   }

   // Write out histograms
   TvIP->Write();
   LtIP->Write();
   Phi0->Write();
   CoTanT->Write();
   QdPT->Write();
   InvPtPhi->Write();
   InvPtA0->Write();
   PhiA0->Write();
   CoTTZ0->Write();
   EtaValues->Write();
   PtValues->Write();

   // Close data and root files
   outFile.Close();
   fclose(fp);

}
