//========================================================================
//  An example of a WMassGen class is given below:                          //
//========================================================================
// HTP modified 6/3/2000 to use PYTHIA6

#ifndef WMassGen_H
#define WMassGen_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// WMassGen                                                             //
//                                                                      //
// This class organize generation of W events                           //
//                                                                      //
//////////////////////////////////////////////////////////////////////////
//
class TH1F;
class TPythia;
class ATLFast;
class ATLFMisc;

class WMassGen {

  protected:
   TH1F             *m_hist1;       //pointer to histogram
   TH1F             *m_hist2;       //pointer to histogram
   TH1F             *m_hist3;       //pointer to histogram
   TH1F             *m_hist4;       //pointer to histogram
  public:
               WMassGen();
   void        Init();
   void        FillHistograms();
   void        StoreHistograms();
   void        Filter();
   void        SetRunParameters();
   void        SetMakerParameters();

};
#endif

//_________________________________________________________________________
WMassGen::WMassGen() 
{
  // dummy
}
//_________________________________________________________________________
void WMassGen::Init() 
{
   // set parameters for a job

   SetRunParameters();    //change default parameters
   SetMakerParameters();  //change default parameters

   //  Create a few histogram

   m_hist1 = new TH1F("hist1","Multiplicity of isolated electrons",20,0,20);
   m_hist2 = new TH1F("hist2","Multiplicity of isolated muons",20,0,20);  
   m_hist3 = new TH1F("hist3","Multiplicity of all-jets",20,0,20);
   m_hist4 = new TH1F("hist4","Filter responds ",10,0,10);
}


//_________________________________________________________________________
void WMassGen::FillHistograms()
{
   ATLFMisc *misc = (ATLFMisc*)gATLFast->MiscMaker()->Fruits();
   Int_t nele=misc->Nelectrons();
   Int_t nmuo=misc->Nmuons();
   Int_t njet=misc->Nalljets();

   m_hist1->Fill(nele+0.01);
   m_hist2->Fill(nmuo+0.01);
   m_hist3->Fill(njet+0.01);
}
//_________________________________________________________________________
void WMassGen::Filter(Int_t &kFilter)
{

   gATLFast->ClusterMaker()->Save(0);
   ATLFMisc *misc = (ATLFMisc*)gATLFast->MiscMaker()->Fruits();
   Int_t nele=misc->Nelectrons();
   Int_t nmuo=misc->Nmuons();
   Int_t njet=misc->Nalljets();
   if(  njet >=2 ) kFilter = 1;
   if(  (nele+nmuo) >= 1 ) kFilter = 2;
   m_hist4->Fill(kFilter + 0.01);

}

//_________________________________________________________________________
void WMassGen::SetRunParameters()
{
   // change ATLfast default parameters

   // W+jet production selected
   
   gATLFast->MCMaker()->SetProductionMode(7);
   TPythia *pythia=new TPythia();
   gATLFast->MCMaker()->SetGenerator(pythia);   
   pythia->SetMSEL(0); //htp set processes myself...  
   //pythia->SetMSUB(2,1);
   //pythia->SetMSUB(16,1);
   //pythia->SetMSUB(31,1);
   //pythia->SetMSUB(25,1);  // ff -> WW
   cout << "Higgs Mass before set" << pythia->GetPMAS(25,1)<<endl;

   pythia->SetPMAS(25,1,160.0);//set higgs mass
   //removed for the purpose of replicating Rob's results
   //pythia->SetMSUB(8,1); // WW -> H 

   //set ff->ffH,ffH via W,Z fusion as per RH
    pythia->SetMSUB(123,1);
   pythia->SetMSUB(124,1);//ww fusion
   
   pythia->Lulist(13);
   int dstart = pythia->GetMDCY(25,2);
   int dend = dstart+pythia->GetMDCY(25,3);
   cout << "From MDCY: " << dstart << " " << dend << endl;
   cout << "23Mass" << pythia->GetPMAS(23,1)<<endl;
   cout << "24Mass" << pythia->GetPMAS(24,1)<<endl;
   cout << "25Mass" << pythia->GetPMAS(25,1)<<endl;

   for(int i=dstart; i < dend ; i++)
     {if (TMath::Abs(pythia->GetKFDP(i,1)==24)) 
	  {pythia->SetMDME(i,1,1);
	  //htp turn its branching ratio to 1.0 as well following RH
          pythia->SetBRAT(i,1.0);
	  cout << "turned on:" << i <<endl;} else 
          {pythia->SetMDME(i,1,0);
          //htp and again following RH
          pythia->SetBRAT(i,0.0);
	  cout << "turned off:" << i << endl;}
     }

   //htp we seem to be having LUSHOW infinite loop problems- try replicating
   // exactly what RH does, ugly though it may be..
   /*
      for(int j=pythia->GetMDCY(24,2);j < pythia->GetMDCY(24,2)+pythia->GetMDCY(24,3); j++)
   { int tempkf = TMath::Abs(pythia->GetKFDP(j,1));
       if (tempkf == 11 || tempkf==12 ) 
       {pythia->SetMDME(j,1,4);} else
             { if (tempkf == 13 || tempkf==14 ) 
                 {pythia->SetMDME(j,1,5);}
             else
   		{pythia->SetMDME(j,1,0);}}     
   }

   */
   //RH dabbling with branching ratios
   pythia->SetMDME(188,1,1.0);
   pythia->SetMDME(189,1,1.0);
   pythia->SetBRAT(188,0.5);
   pythia->SetBRAT(189,0.5);



   //pythia->SetMSUB(118,1); // gg -> WW ARGH! Not implemented yet!
   // random number and hadronization option   
   gATLFast->MCMaker()->SetHadronisationMode(3);
   gATLFast->MCMaker()->SetRndNumber(51111);
   // others option as default values   


   // RMS - Alterations to pythia parameters as per fortran version
   pythia->SetMDCY(6,1,1);
   pythia->SetMSTP(48,1);
   pythia->SetMSTU(22,100);
   pythia->SetMSTU(112,4);
   pythia->SetMSTU(118,5);
   pythia->SetPMAS(6,1,175.0); // Mass of top
   pythia->SetMSTJ(11,3);  //Peterson fragmentation function for b and c:
   pythia->SetPARJ(54,-0.07);
   pythia->SetPARJ(55,-0.006);
   pythia->SetPARU(112,0.19000);
   pythia->SetPARU(117,0.14297);
   pythia->SetPARU(118,0.11674);
   pythia->SetPARF(106,175.0);
   pythia->SetPARF(107,250.0);
   pythia->SetPARF(108,250.0);

// default (KEYHAD=3):

   pythia->Initialize("CMS","p","p",14000);

}
//_________________________________________________________________________
void WMassGen::SetMakerParameters()
{
   // change ATLfast default parameters
   // decide which branch to store
   gATLFast->SetSmearing(1);
   gATLFast->SetTrackFinding(1);
   gATLFast->ClusterMaker()->SetGranBarrelEta(0.1);
   gATLFast->PhotonMaker()->SetMinPT(5.);
  //gATLFast->TriggerMaker()->SetMuoEtaCoverage(2.5);
   gATLFast->SetSmearMuonOpt(2);
   gATLFast->ClusterMaker()->Save(1);
   //gATLFast->TrackMaker()->Save(1);
   gATLFast->MCMaker()->Save(2);
}
//_________________________________________________________________________
void WMassGen::StoreHistograms()
{
  Int_t i;
  
  // Draw histograms
  
  TCanvas *wmc = new TCanvas("wmc","W mass",200,10,600,480);
  wmc->Divide(2,2);
  wmc->cd(1);   m_hist1->Draw();
  wmc->cd(2);   m_hist2->Draw();
  wmc->cd(3);   m_hist3->Draw();
  wmc->cd(4);   m_hist4->Draw();
  
  wmc->Update();
  
  // Store WMass.eps file
  
  wmc->Print("WMass_filter.eps");
  
  // Store histograms on file
  
  TFile f2("WMass_filter.root","RECREATE","WMass exercise");
  
  m_hist1->Write();
  m_hist2->Write();
  m_hist3->Write();
  m_hist4->Write();
  
  f2->Close();
  
}











