C -------------------------------------------------------------------           
C Routines to read in Genz Data
C 
C  - genzinit  :  initialise genz and open input file 
C  - genzread  :  read data  
C  - genzend   :  shut down
C
C Original Version                                 R.DeWolf 05/03/91
C ---------------------------------------------------------------------

      Subroutine genzinit
 
      INTEGER MEMGEN,MEMPAW
      PARAMETER (MEMPAW =400000)
      PARAMETER (MEMGEN =300000)

      REAL GJUNK,HJUNK
      COMMON/PAWC/HJUNK(MEMPAW)
      COMMON/GNCSTO/GJUNK(MEMGEN)

      CALL INITC

      CALL HLIMIT(MEMPAW)
      CALL GNZINIT(-280000)                                        
                                                                      
      print *,'Calling gnzopen.....ZEBRA.P' 
      CALL GNZOPEN(25,'INP1', 'ZEBRA.P') 

      END

C --------------------------------------------------------------------------    

      Subroutine genzend()
 
      CALL GNZEND

      END

C ---------------------------------------------------------------------------

      Subroutine genzread()

C 
C  GNZREAD :Read a "Pseudo-event" (could be event, run header, or both)
C           called once per evt to give run#, SOR, EOR.. 
C  GNZGETG returns the # of overlapped evts in 1 evt : NEVT=1 no pile-up
C  NEVT >1    first event : signal, others : minimum-bias
C  Loop over particles
C  loop on each paricle of the given evt
C        ista=1  stabel particle, p(4) 4 momentum, icode particle code,
C        imoth(2) 1 =position of the mother; v(3) vertex particle
C
C V comes out in m, and we neei in  mm (as PYTHIA standard) multiply by 1000
C
C ------------------------------------------------------------------------- 
 
      INTEGER IP, IE
      INTEGER IRET,NEVTS,NPART,NKEEP,ICODE,ISTAT
      INTEGER IEVT,IRUN,ISOR
      INTEGER JIDGN,IGTIM,IGDAT,IDATG
      CHARACTER*4 CMACH,CGENE
      REAL VRGEN,VRGNZ,VRZEB
      INTEGER IWTFL
      REAL VEVT(4),WEIGH,TIME
      INTEGER IMOTH(2),IDAU1
      REAL P(4),V(3),AMASS

C.....lujets common
      INTEGER NLUJ, KLUJ
      REAL PLUJ, VLUJ     
      COMMON/LUJETS/NLUJ,KLUJ(4000,5),PLUJ(4000,5),VLUJ(4000,5) 

 5    CALL GNZREAD(IRET)

      IF(IRET.EQ.5) RETURN         ! End of Job
      IF(IRET.LE.-1) RETURN        ! Error

      IF(IRET.EQ.1)THEN
         WRITE(6,3020)
         ISOR = 1
         GOTO 5
      ELSE IF(IRET.EQ.2)THEN
         WRITE(6,3021)
         RETURN
C
      ELSEIF(IRET.EQ.0)THEN
C
       IF(ISOR.EQ.1)THEN
          ISOR = 0
C
          CALL GNZGETR(1,JIDGN,IRUN,IGTIM,IGDAT,CMACH,CGENE,
     +                 VRGEN,IDATG,VRGNZ,VRZEB)
C
          WRITE(6,1021)IRUN,CGENE
          WRITE(6,1025)JIDGN,IGDAT,IGTIM,CMACH
          WRITE(6,1026)VRGEN,IDATG,VRGNZ,VRZEB
C
       ENDIF

         CALL GNZGETG(1,NEVTS)
         IF(NEVTS.EQ.1) THEN
           IE = 1
         ELSE
           print *,'More than one event in the gate...ciao'
           RETURN
         ENDIF

C No events?
         IF(NEVTS.LE.0) return
         NKEEP = 0

c  per evt return number of particles...
            CALL GNZGETE(1,IE,NPART,IRUN,IEVT,CGENE,VEVT,IWTFL,WEIGH)
            IF(NPART.EQ.0) return
            IF(IE.EQ.1) THEN

              NLUJ=NPART  
              DO 20 IP=1,NPART
               CALL GNZGETP(1,IE,IP,ISTAT,ICODE,P,AMASS,IMOTH,
     +         TIME,IDAU1,V)
               IF(ISTAT.NE.1) ISTAT=11
               KLUJ(IP,1)=ISTAT
               KLUJ(IP,2)=ICODE
               KLUJ(IP,3)=IMOTH(1)
               KLUJ(IP,4)=IDAU1
               PLUJ(IP,1)=P(1)
               PLUJ(IP,2)=P(2)
               PLUJ(IP,3)=P(3)
               PLUJ(IP,4)=P(4)
               PLUJ(IP,5)=AMASS
               VLUJ(IP,1)=V(1)*1000.
               VLUJ(IP,2)=V(2)*1000.
               VLUJ(IP,3)=V(3)*1000.
               VLUJ(IP,4)=V(4)
               VLUJ(IP,5)=TIME
   20         CONTINUE
            ENDIF
c end loop on particles given event

C Get last daughter
       DO 27 I=1,NPART
       IDAU1 = KLUJ(I,4)
       IDAU2 = IDAU1
       IF (IDAU1.GT.0) THEN
        DO 35 J=IDAU1+1,NPART
        IF (KLUJ(J,3).EQ.I) THEN
         IDAU2 = J
        ELSE
         GO TO 15
        END IF
 35     CONTINUE
       END IF
 15    KLUJ(I,5) = IDAU2
 27   CONTINUE

      ENDIF

C Formats..
C
 3020 FORMAT(1X,'START of run read' )
 3021 FORMAT(1X,'END of run read' )
 1021 FORMAT(1X,'Header for run',I10,' read. Primary generator= ',A4)
 1025 FORMAT(1X,'Job ID = ',I10,'  Run Date,Time = ',I10,2X,I10,/,
     +1X,'Machine = ',A4)
 1026 FORMAT(1X,'Generator version  = ',F8.4,'  Date = ',I10,/,
     +       1X,'GENZ      version  = ',F8.4,/,
     +       1X,'Zebra     version  = ',F8.4)


       END

C ----------------------------------------------------------------------
      SUBROUTINE GNZOPEN(LUN,CSTRM,CFILE)
C
C Input:  LUN   - unit number to use
C         CSTRM - stream name: 'INP1' 'INP2' or 'OUTP'
C         CFILE - name of file to open
C
C Output: none
C
C Original Version:                    R. DeWolf   04/11/92 16.21.38  *
C
C Modifications:
C
C-------------------------------------------------------------------------
*KEEP,TYPING.
*KEEP,GNCPAR.
C      INTEGER NGNIPA,NGNCPA,NGNIP2,NGNCP2,NGNRPA,NGNRP2 
C Number of parameters NGNIPA; Number printed and modifiable, NGNIP2
      PARAMETER (NGNIPA = 21)
      PARAMETER (NGNIP2 = 14)
      INTEGER IGNPAR
      COMMON/GNCIPA/IGNPAR(NGNIPA)
      EQUIVALENCE (IGNPAR(1),LGNPRI)
      EQUIVALENCE (IGNPAR(2),LRECI1)
      EQUIVALENCE (IGNPAR(3),LRECI2)
      EQUIVALENCE (IGNPAR(4),LRECOU)
      EQUIVALENCE (IGNPAR(5),LVMZEB)
      EQUIVALENCE (IGNPAR(6),LVGNMZ)
      EQUIVALENCE (IGNPAR(7),LVGNFZ)
      EQUIVALENCE (IGNPAR(8),LVGNPR)
      EQUIVALENCE (IGNPAR(9),IGNRUN)
      EQUIVALENCE (IGNPAR(10),IGNGT0)
      EQUIVALENCE (IGNPAR(11),IGNJID)
      EQUIVALENCE (IGNPAR(12),IGNUWD)
      EQUIVALENCE (IGNPAR(13),IGNGVR)
      EQUIVALENCE (IGNPAR(14),IGNGDT)
      EQUIVALENCE (IGNPAR(15),IGNSTA)
      EQUIVALENCE (IGNPAR(16),LFZIN1)
      EQUIVALENCE (IGNPAR(17),LFZIN2)
      EQUIVALENCE (IGNPAR(18),LFZOUT)
      EQUIVALENCE (IGNPAR(19),IGNMOD)
      EQUIVALENCE (IGNPAR(20),IGNWMI)
      EQUIVALENCE (IGNPAR(21),IGNWMX)
C
      INTEGER IGNSTA,LFZIN1,LRECI1,LFZIN2,LRECI2,LFZOUT,LRECOU,LGNPRI
      INTEGER LVMZEB,LVGNMZ,LVGNFZ,LVGNPR,IGNRUN,IGNJID,IGNGT0,
     +IGNUWD,IGNGVR,IGNGDT
      INTEGER IGNMOD,IGNWMI,IGNWMX
C Character parameters:
C Number of parameters NGNC*KEEP,QUEST.
C Standard Zebra common
      COMMON/QUEST/IQUEST(100)
      INTEGER IQUEST
*KEND.
C Arguments:
      INTEGER LUN
      CHARACTER*(*) CFILE,CSTRM
C Local:
      INTEGER LREC,IOPER,MXCLEN,LENC
      CHARACTER*(3) CSTAT
      CHARACTER*(2) IOFLG
      CHARACTER*(5) CFZOP
      PARAMETER (MXCLEN = 40)
      CHARACTER*(MXCLEN) CBLAN
      SAVE CBLAN
      DATA CBLAN/' '/
C
      CSTAT = 'UNK'
      LREC = 0
C
C Allow ZFATAL to handle error returns:
C Output stream:
C
      IF(CSTRM.EQ.'OUTP')THEN
         IOPER = 0
         LFZOUT = LUN
         LREC = LRECOU
         CSTAT = 'NEW'
* Quick Fix - Do NOT distribute (look what happened last time).
         CFZOP = 'XOL '
         IOFLG = 'w '
         WRITE(LGNPRI,10150)LUN
      ELSE IF(CSTRM.EQ.'INP1')THEN
         print *,'CSTRM.EQ.INP1'
         LFZIN1 = LUN
         IOPER = 0
         LREC = LRECI1
         CSTAT = 'OLD'
         CFZOP = 'XL '
         IOFLG = 'r '
         WRITE(LGNPRI,10500)LUN
      ELSE IF(CSTRM.EQ.'INP2')THEN
         LFZIN2 = LUN
         IOPER = 0
         LREC = LRECI2
         CSTAT = 'OLD'
         CFZOP = 'XL '
         IOFLG = 'r '
         WRITE(LGNPRI,10500)LUN
      ENDIF
C
      print *,'lrec = ',LREC
      IF(LREC.NE.0)THEN
C
C See if there was a file name given:
         print *,'filename'
         LENC = LEN(CFILE)
         IF((CFILE(1:LENC).EQ.CBLAN(1:LENC)).AND.(LENC.LE.MXCLEN))THEN
C No file name given:
            print *,'no filename'
            WRITE(LGNPRI,10200)LREC
            OPEN(UNIT=LUN,IOSTAT=IOPER,STATUS=CSTAT,FORM='UNFORMATTED')
         ELSE
            WRITE(LGNPRI,10300)LREC,CFILE
           IF ( INDEX(CFZOP(1:3),'L') .NE.0 ) THEN
            IQUEST(10)=1
            CALL CFOPEN(IQUEST(1),0,LREC,IOFLG,0,
     +           CFILE(1:LENOCC(CFILE)),IOPER)
           ELSE
            OPEN(UNIT=LUN,IOSTAT=IOPER,STATUS=CSTAT,FORM='UNFORMATTED',
     +      FILE=CFILE)
           ENDIF
         ENDIF
C
         IF(IOPER.NE.0) WRITE(LGNPRI,10400)IOPER,LUN
         CALL FZFILE(LUN,LREC,CFZOP)
         CALL FZLOGL(LUN,LVGNFZ)

      ELSE
         WRITE(LGNPRI,10000)CSTRM
      ENDIF
10000 FORMAT(1X,'GNZOPEN: Unrecognized stream key :',A,'  OPEN FAILS')
10100 FORMAT('0','         OPEN of unit',I4,' for OUTPUT')
10150 FORMAT('0','         Clibrary OPEN of unit',I4,' for OUTPUT')
10200 FORMAT(1X,'         RECL=',I6,'   No filename given ')
10300 FORMAT(1X,'         RECL=',I6,'   Filename = ',A)
10400 FORMAT(1X,' **** ERROR! IOSTAT=',I5,' opening file for unit',I4)
10500 FORMAT('0','         OPEN of unit',I4,' for INPUT')
      END

