//========================================================================
//  An example of a WMassGen class is given below:                          //
//========================================================================

#ifndef WMassGen_H
#define WMassGen_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// WMassGen                                                                //
//                                                                      //
// This class organize generation, simulation and analysis of W events  //
//                                                                      //
//////////////////////////////////////////////////////////////////////////
//
class TH1F;
class TPythia;
class ATLFast;
class ATLFMisc;

class WMassGen {

  protected:
   TH1F             *m_hist1;       //pointer to histogram
   TH1F             *m_hist2;       //pointer to histogram
   TH1F             *m_hist3;       //pointer to histogram
   TH1F             *m_hist4;       //pointer to histogram
  public:
               WMassGen();
   void        Init();
   void        FillHistograms();
   void        StoreHistograms();
   void        Filter();
   void        SetRunParameters();
   void        SetMakerParameters();

}
#endif

//_________________________________________________________________________
WMassGen::WMassGen() 
{
  // dummy
}
//_________________________________________________________________________
void WMassGen::Init() 
{
   // set paramters for a job

   SetRunParameters();    //change default parameters
   SetMakerParameters();  //change default parameters

   //  Create a few histogram

   m_hist1 = new TH1F("hist1","Number of electrons per event",20,0,20);
   m_hist2 = new TH1F("hist2","Number of isolated muons",20,0,20);  
   m_hist3 = new TH1F("hist3","Number of all-jets",20,0,20);
   m_hist4 = new TH1F("hist4","Filter responds ",10,0,10);
}


//_________________________________________________________________________
void WMassGen::FillHistograms()
{
   ATLFMisc *misc = (ATLFMisc*)gATLFast->MiscMaker()->Fruits();
   Int_t nele=misc->Nelectrons();
   Int_t nmuo=misc->Nmuons();
   Int_t njet=misc->Nalljets();

   m_hist1->Fill(nele+0.01);
   m_hist2->Fill(nmuo+0.01);
   m_hist3->Fill(njet+0.01);
}
//_________________________________________________________________________
void WMassGen::Filter(Int_t &kFilter)
{

   gATLFast->ClusterMaker()->Save(0);
   ATLFMisc *misc = (ATLFMisc*)gATLFast->MiscMaker()->Fruits();
   Int_t nele=misc->Nelectrons();
   Int_t nmuo=misc->Nmuons();
   Int_t njet=misc->Nalljets();
   if(  njet >=2 ) kFilter = 1;
   if(  (nele+nmuo) >= 1 ) kFilter = 2;
   m_hist4->Fill(kFilter + 0.01);

}

//_________________________________________________________________________
void WMassGen::SetRunParameters()
{
   // change ATLfast default parameters

   // W+jet production selected
   gATLFast->MCMaker()->SetProductionMode(0);
   TPythia *pythia=new TPythia();
   pythia->SetMSUB(2,1);
   pythia->SetMSUB(16,1);
   pythia->SetMSUB(31,1);
   //random number and hadronization option   
   gATLFast->MCMaker()->SetHadronisationMode(3);
   gATLFast->MCMaker()->SetRndNumber(51111);
   //others option as default values   
}
//_________________________________________________________________________
void WMassGen::SetMakerParameters()
{
   // change ATLfast default parameters
   gATLFast->SetSmearMuonOpt(2);
   gATLFast->ClusterMaker()->Save(0);
   gATLFast->TrackMaker()->Save(0);
   gATLFast->MCMaker()->Save(1);
}
//_________________________________________________________________________
void WMassGen::StoreHistograms()
{
   Int_t i;

// Draw histograms

   c1 = new TCanvas("c1"," ",200,10,600,480);
   c1->Divide(2,2);
   c1->cd(1);   m_hist1->Draw();
   c1->cd(2);   m_hist2->Draw();
   c1->cd(3);   m_hist3->Draw();
   c1->cd(4);   m_hist4->Draw();

// Store WMass.eps file

   c1->Print("WMass_filter.eps");

// Store histograms on file

   TFile f2("WMass_filter.root","RECREATE","WMass exercise");

    m_hist1->Write();
    m_hist2->Write();
    m_hist3->Write();
    m_hist4->Write();

   f2->Close();

}






