//========================================================================
//  An example of a User class is given below:                          //
//========================================================================

#ifndef user_H
#define user_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// User                                                                 //
//                                                                      //
// Example of a user class to perform user specific tasks when running  //
// the ATLfast program.                                                 //
//                                                                      //
// This class illustrates:                                              //
//   - How to set run parameters                                        //
//   - How to create and fill histograms                                //
//                                                                      //
//////////////////////////////////////////////////////////////////////////
//
class TH1F;
class ATLFast;
class ATLFMisc;

class User {

  protected:
   TH1F             *m_hist1;       //pointer to histogram
   TH1F             *m_hist2;       //pointer to histogram
   TH1F             *m_hist3;       //pointer to histogram
  public:
               User();
   void        Init();
   void        FillHistograms();
   void        Store Histograms();
   void        Filter();
   void        SetRunParameters();
   void        SetMakerParameters();

}
#endif

//_________________________________________________________________________
User::User() 
{
  // dummy
}
//_________________________________________________________________________
void User::Init() 
{
   // set paramters for a job

   SetRunParameters();    //change default parameters
   SetMakerParameters();  //change default parameters

   //  Create a few histogram

   m_hist1 = new TH1F("hist1","Number of electrons per event",20,0,20);
   m_hist2 = new TH1F("hist2","Number of isolated muons",20,0,20);  
   m_hist3 = new TH1F("hist3","Number of b-jets",20,0,20);
}


//_________________________________________________________________________
void User::FillHistograms()
{
   ATLFMisc *misc = (ATLFMisc*)gATLFast->MiscMaker()->Fruits();
   Int_t nele=misc->Nelectrons();
   Int_t nmuo=misc->Nmuons();
   Int_t nbjet=misc->Nbjets();

   m_hist1->Fill(nele+0.01);
   m_hist2->Fill(nmuo+0.01);
   m_hist3->Fill(nbjet+0.01);
}
//_________________________________________________________________________
void  User::Filter(Int_t &kFilter)
{

   gATLFast->ClusterMaker()->Save(0);
   ATLFMisc *misc = (ATLFMisc*)gATLFast->MiscMaker()->Fruits();
   Int_t nbjet=misc->Nbjets();
   if(nbjet >=2) kFilter = 1;

}

//_________________________________________________________________________
void User::SetRunParameters()
{
   // change ATLfast default parameters

   gATLFast->MCMaker()->SetProductionMode(7);
   gATLFast->MCMaker()->SetHadronisationMode(3);
   gATLFast->MCMaker()->SetRndNumber(51111);
}
//_________________________________________________________________________
void User::SetMakerParameters()
{
   // change ATLfast default parameters
   gATLFast->SetSmearMuonOpt(2);
}
//_________________________________________________________________________
void User::StoreHistograms()
{
   Int_t i;

// Draw histograms

   c1 = new TCanvas("c1"," ",200,10,600,480);
   c1->Divide(2,2);
   c1->cd(1);   m_hist1->Draw();
   c1->cd(2);   m_hist2->Draw();
   c1->cd(3);   m_hist3->Draw();

// Store user.eps file

   c1->Print("user.eps");

// Store histograms on file

   TFile f2("user.root","RECREATE","user exercise");

    m_hist1->Write();
    m_hist2->Write();
    m_hist3->Write();

   f2->Close();

}






