#ifndef __CINT__

#include <stdlib.h>
#include <ctype.h>

#ifdef PYTHIA6
#include <TPythia6.h>
#else
#include <TPythia.h>
#endif

#include <TROOT.h>
#include <TApplication.h>
#include <TFile.h>
#include "ATLFast.h"
#include "ATLFMCMaker.h"

#endif


// example of main program/macro to run ATLFast

void umain(Int_t prodmode=1, Int_t nevents=1000)
{
#ifdef __CINT__
   gROOT->Reset();
   gROOT->Macro("$ATLFAST/loadlibs");
#endif

   // Open the root output file
   TFile file("atlfast.root","recreate","ATLFast root file",2);
   
   ATLFast atlfast("atlfast");     // create main object to run atlfast

   atlfast.MCMaker()->SetProductionMode(prodmode);

   gROOT->Macro("user.C");  // execute user macro to change some parameters

   atlfast.Init();      // Initialise event (maker histograms,etc)
   atlfast.MakeTree();  // Create the Root tree

#ifdef PYTHIA6
   TPythia6 *pythia = (TPythia6 *) gATLFast->MCMaker()->Generator();
#else
   TPythia *pythia = (TPythia *) gATLFast->MCMaker()->Generator();
#endif
   pythia->Pystat(2);
   atlfast.PrintInfo();

   for (Int_t i=0; i<nevents; i++) {
      if (i%20 == 0) printf("In loop:%d\n",i);
      atlfast.Make(i);       // Generate and Reconstruct event
      atlfast.FillTree();
      if (i < 1) pythia->Lulist(1);
      atlfast.Clear();       // Clear reconstructed event lists
   }

   atlfast.Finish();
   // save objects in Root file
//   atlfast.Tree()->Print();
   atlfast.Write();  //save main atlfast object (and run parameters)
//   file.Write();     // save histograms and tree
}


#ifndef __CINT__

TROOT root("atlfast", "Atlfast", 0);

int main(int argc, char **argv)
{
   TApplication theApp("Atlfast", &argc, argv);

   int prodmode = 1;
   int nevt     = 1000;

   if (argv[1] && isdigit(*argv[1]))
      prodmode = atoi(argv[1]);
   if (argv[2] && isdigit(*argv[2]))
      nevt = atoi(argv[2]);
 
   umain(prodmode, nevt);

   return 0;
}

#endif
