//========================================================================
//  An example of a WMassAnal class is given below:                     //
//========================================================================

#ifndef WMassAnal_H
#define WMassAnal_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// WMassAnal                                                            //
//                                                                      //
// This class organizes simulation and analysis of W events             // 
//  void       Filter();                                                //
//  void       SetRunParameters();                                      //
//  void       SetMakerParameters();                                    //
//                                                                      //
//                                                                      //
//////////////////////////////////////////////////////////////////////////
//
class TH1F;
class TPythia;
class ATLFast;
class ATLFMisc;

class WMassAnal {

  protected:
   TH1F       *m_hist1;       //pointer to histogram
   TH1F       *m_hist2;       //pointer to histogram
   TH1F       *m_hist3;       //pointer to histogram
   TH1F       *m_hist11;      //pointer to histogram
   TH1F       *m_hist12;      //pointer to histogram
   TH1F       *m_hist13;      //pointer to histogram
  public:
               WMassAnal() {;}
   void        Init();
   void        Analyse();
   void        Calibrate();
   void        AnalMjj();
   void        Finish();
}
#endif

//_________________________________________________________________________
void WMassAnal::Init() 
{
   //  Create a few histograms

   m_hist1 = new TH1F("hist1","Number of electrons per event",20,0,20);
   m_hist2 = new TH1F("hist2","Number of isolated muons",20,0,20);  
   m_hist3 = new TH1F("hist3","Number of all-jets",20,0,20);

   m_hist11 = new TH1F("hist11","Mass jj crude",50,0,150);
   m_hist12 = new TH1F("hist12","Mass jj",50,0,150);
   m_hist13 = new TH1F("hist13","Mass lnu",50,0,150);
}


//_________________________________________________________________________
void WMassAnal::Analyse()
{
   ATLFMisc *misc = (ATLFMisc*)gATLFast->MiscMaker()->Fruit();
   Int_t nele=misc->Nelectrons();
   Int_t nmuo=misc->Nmuons();
   Int_t njet=misc->Nalljets();

   m_hist1->Fill(nele);
   m_hist2->Fill(nmuo);
   m_hist3->Fill(njet);
   
   if( njet >= 2 ) {
      AnalMjj(m_hist11);
      Calibrate();
      AnalMjj(m_hist12);
   }

   if( (nele+nmuo) == 1 ) {
      AnalMlnu (m_hist13);
   }

}

//_________________________________________________________________________
void WMassAnal::AnalMjj(TH1F *hist)
{

  Int_t i,k ;
  Float_t pjx1,pjy1,pjz1,pje1;
  Float_t pjx2,pjy2,pjz2,pje2;
  Float_t massjj;

  TClonesArray *jets = gATLFast->JetMaker()->Fruits();
  ATLFJet *jet1, *jet2;
  Int_t njets    = jets->GetEntriesFast();

  for (i=0;i<njets-1;i++) {
     jet1 = (ATLFJet*)jets->UncheckedAt(i);
     pjx1 = jet1->PT()*TMath::Cos(jet1->Phi());
     pjy1 = jet1->PT()*TMath::Sin(jet1->Phi());
     pjz1 = jet1->PT()*TMath::SinH(jet1->Eta());
     pje1 = jet1->PT()*TMath::CosH(jet1->Eta());
     for (k=i+1;k<njets;k++) {
        jet2 = (ATLFJet*)jets->UncheckedAt(k);
        pjx2 = jet2->PT()*TMath::Cos(jet2->Phi());
        pjy2 = jet2->PT()*TMath::Sin(jet2->Phi());
        pjz2 = jet2->PT()*TMath::SinH(jet2->Eta());
        pje2 = jet2->PT()*TMath::CosH(jet2->Eta());
        massjj = (pje1+pje2)*(pje1+pje2) -(pjx1+pjx2)*(pjx1+pjx2)
                -(pjy1+pjy2)*(pjy1+pjy2) -(pjz1+pjz2)*(pjz1+pjz2);
        if (massjj > 0) massjj = TMath::Sqrt(massjj);
        else              massjj = 0;
        hist->Fill(massjj);
     }
  }
}

//_________________________________________________________________________
void WMassAnal::AnalMlnu(TH1F *hist)
{
  Float_t const aamw = 80.0;
  Float_t pxlep,pylep,pzlep,eelep;
  Float_t pxnu,pynu,pznu1,pznu2,eenu1,eenu2;
  Float_t pxw,pyw,pzw1,pzw2,eew1,eew2;
  Float_t masswln;
  Float_t A,B,aa,bb,C,cc,D,delta;

  TClonesArray *electrons = gATLFast->ElectronMaker()->Fruits();
  ATLFElectron *ele;

  TClonesArray *muons = gATLFast->MuonMaker()->Fruits();
  ATLFMuon *muo;
  
  ATLFMisc *misc = (ATLFMisc*)gATLFast->MiscMaker()->Fruit();
  Int_t nele = misc->Nelectrons();
  Int_t nmuo = misc->Nmuons();

   // looking for lepton from W-->l nu
  if( nele == 1 ) {
     ele  = (ATLFElectron*)electrons->UncheckedAt(0);
     pxlep= ele->PT()*TMath::Cos(ele->Phi());
     pylep= ele->PT()*TMath::Sin(ele->Phi());
     pzlep= ele->PT()*TMath::SinH(ele->Eta());
     eelep= ele->PT()*TMath::CosH(ele->Eta());
  } elseif( nmuo == 1 ) {
     muo  = (ATLFMuon*)muons->UncheckedAt(0);
     pxlep= muo->PT()*TMath::Cos(muo->Phi());
     pylep= muo->PT()*TMath::Sin(muo->Phi());
     pzlep= muo->PT()*TMath::SinH(muo->Eta());
     eelep= muo->PT()*TMath::CosH(muo->Eta());
  }
   // reconstruct W -> l nu
  masswln = 0.;
  pxnu  = misc->pxmiss();
  pynu  = misc->pymiss();
  pxw   = pxlep + pxnu;
  pyw   = pylep + pxnu;
  A     = aamw*aamw + pxw*pxw +pyw*pyw-eelep*eelep;
  B     = 0.5*(A-pxnu*pxnu-pynu*pynu+pzlep*pzlep);
  aa    = B*B-((eelep*eelep)*(pxnu*pxnu+pynu*pynu));  
  bb    = eelep*eelep-pzlep*pzlep;
  C     = -aa/bb;
  cc    = 2*B*pzlep;
  D     = cc/bb;
  delta = D*D-4*C;
  if( delta > 0 ) {
     delta = TMath::Sqrt(D*D-4*C);
     pznu1 = 0.5*(D-delta);
     pznu2 = 0.5*(D+delta);
     pzw1  = pzlep+pznu1;
     pzw2  = pzlep+pznu2;
     eew1  = eelep+TMath::Sqrt(pxnu*pxnu+pynu*pynu+pznu1*pznu1);
     eew2  = eelep+TMath::Sqrt(pxnu*pxnu+pynu*pynu+pznu2*pznu2);
     masswln = eew1*eew1 - pxw*pxw - pyw*pyw - pzw1*pzw1;
     if (masswln > 0) masswln = TMath::Sqrt(masswln);
     else             masswln = 0;
  }
  if(masswln == 0) masswln = masswln + 0.01;
  hist->Fill(masswln);
}

//_________________________________________________________________________
void WMassAnal::Calibrate()
{
  Int_t k ;
  Float_t pT ;
  const Float_t kFac = 1.2;
  TClonesArray *jets = gATLFast->JetMaker()->Fruits();
  ATLFJet *jet, *jetcru;
  Int_t njets = jets->GetEntriesFast();

  for (k=0;k<njets;k++) {
     jet = (ATLFJet*)jets->UncheckedAt(k);
     jetcru = jet;
     pT =  jet->PT();
     jet->SetPT(pT*kFac);
     printf("pT: %f,%f,%f\n",pT,jet->PT(),jetcru->PT());
  }
}

//_________________________________________________________________________
void WMassAnal::Finish()
{
// Draw histograms
  c1 = new TCanvas("c1"," ",200,10,600,480);
  c1->Divide(2,2);
  c1->cd(1);   m_hist11->Draw();
  c1->cd(2);   m_hist12->Draw();
  c1->cd(3);   m_hist13->Draw();

// Create Postscript WMass.eps file
  c1->Print("WMass_anal.eps");

// Store histograms on file

  TFile f2("WMass_anal.root","RECREATE","WMass exercise");

  m_hist1->Write();
  m_hist2->Write();
  m_hist3->Write();
  m_hist11->Write();
  m_hist12->Write();
  m_hist13->Write();
}






